% *******************************************************
%	IHRS design and optimization using Genetic Algorithms 
%	Building the initial data input matrix.
% *******************************************************
%	16.01.2000 - last : 31.01.2000 by OW|BR
%	DataInput.m
% *******************************************************

if helppri==1, help DataInput; end
pausetext = 'Press any key ...';

%	Read in the full_name and code_name string input matrix file (All names)
[ str_name, str_code ] = textread ( 'InputName.dat', '%s %s' );
disp(' ');
if interpri > 1
	disp ( 'Input Name matrix : ' );
	disp(' ');
	disp ( [ str_name, str_code ] );
	pause
end

load InputData.dat;
if interpri > 1
	disp ( 'Input Data matrix : ' );
	disp(' ');
	disp ( InputData );
	disp (pausetext);
	pause
end
%	DataUnits;

disp('');
Message = '			Now the input data are assigned to Stream arrays !';
disp ( Message );
disp('');

%	********************************************
%	Reading and Initialisation of PROCESS STREAM 
%	********************************************
i_index_start = 1;
t_initial = InputData(i_index_start,1);
t_end     = InputData(i_index_start,2);
I         = InputData(i_index_start,3);
J         = InputData(i_index_start,4);
ProcessName = char ( str_name(i_index_start)' );
RunName     = char ( str_code(i_index_start)' );

QC_HC = 0;	%	total cooling need of hot process composite
QC_HC_tot = 0;	%	total cooling need of hot composite, including optional streams (used for settings correct range of HR versus TBCs plot)
for i = 1:I
	ii = i + i_index_start;
	Stream(i).full_name = char ( str_name(ii)' );
	Stream(i).code_name = char ( str_code(ii)' );
	Stream(i).t_start  = InputData(ii,1);
	Stream(i).t_stop   = InputData(ii,2);
	Stream(i).T_S      = InputData(ii,3);
	Stream(i).T_T      = InputData(ii,4);
	Stream(i).CP       = InputData(ii,5);
	Stream(i).h        = InputData(ii,6);
	Stream(i).optional = InputData(ii,7);
   if Stream(i).t_stop > Stream(i).t_start
      Stream(i).Dt = (Stream(i).t_stop - Stream(i).t_start) / 60;
   else
      Stream(i).Dt = (t_end - t_initial - (Stream(i).t_start - Stream(i).t_stop)) / 60;
   end
   Stream(i).CP_Dt = Stream(i).CP * Stream(i).Dt;
   if interpri > 0
		Stream(i)
		disp (pausetext)
		pause
   end
   QC_HC_tot = QC_HC_tot + Stream(i).CP_Dt * (Stream(i).T_S - Stream(i).T_T);
   if Stream(i).optional == 0	%	i.e. process stream, not optional
      QC_HC = QC_HC + Stream(i).CP_Dt * (Stream(i).T_S - Stream(i).T_T);
   end
end
QC_HC
QC_HC_tot

Abs_QH_CC = 0;	%	total heating need of cold process composite
Abs_QH_CC_tot = 0;	%	total heating need of cold composite, including optional streams (used for settings correct range of HR versus TBCs plot
for j = 1+I:I+J
	jj = j + i_index_start;
	Stream(j).full_name = char ( str_name(jj)' );
	Stream(j).code_name = char ( str_code(jj)' );
	Stream(j).t_start  = InputData(jj,1);
	Stream(j).t_stop   = InputData(jj,2);
	Stream(j).T_S      = InputData(jj,3);
	Stream(j).T_T      = InputData(jj,4);
	Stream(j).CP       = InputData(jj,5);
	Stream(j).h        = InputData(jj,6);
   Stream(j).optional = InputData(jj,7);
   if Stream(j).t_stop > Stream(j).t_start
      Stream(j).Dt = (Stream(j).t_stop - Stream(j).t_start) / 60;
   else
      Stream(j).Dt = (t_end - t_initial - (Stream(j).t_start - Stream(j).t_stop)) / 60;
   end
   Stream(j).CP_Dt = Stream(j).CP * Stream(j).Dt;
   if interpri > 0
		Stream(j)
		disp (pausetext)
		pause
   end
   Abs_QH_CC_tot = Abs_QH_CC_tot + Stream(j).CP_Dt * (Stream(j).T_S - Stream(j).T_T);
   if Stream(j).optional == 0	%	i.e. process stream, not optional
      Abs_QH_CC = Abs_QH_CC + Stream(j).CP_Dt * (Stream(j).T_S - Stream(j).T_T);
   end
end
Abs_QH_CC
Abs_QH_CC_tot

P = I + J;				% number of process streams !

if interpri > 1
		P				% Total number of streams (hot & cold) : P = I + J
		Stream
		disp(' ');
		disp (pausetext);
		pause
	end

%	********************************************
%	Reading and Initialisation of UTILITY STREAM 
%	********************************************
m_index_start = i_index_start + I + J + 1;
M = InputData(m_index_start,1);
N = InputData(m_index_start,2);
O = InputData(m_index_start,3);
K = InputData(m_index_start,4);

for m = 1:M
	mm = m + m_index_start;
	HotUtility(m).full_name = char ( str_name(mm)' );
	HotUtility(m).code_name = char ( str_code(mm)' );
	HotUtility(m).T_in  = InputData(mm,1);
	HotUtility(m).T_out = InputData(mm,2);
	HotUtility(m).h     = InputData(mm,3);
	HotUtility(m).c     = InputData(mm,4);
    if interpri > 0
	HotUtility(m)
	disp (pausetext)
	pause
    end
end
for n = 1:N
	nn = n + M + m_index_start;
	ColdUtility(n).full_name = char ( str_name(nn)' );
	ColdUtility(n).code_name = char ( str_code(nn)' );
	ColdUtility(n).T_in  = InputData(nn,1);
	ColdUtility(n).T_out = InputData(nn,2);
	ColdUtility(n).h     = InputData(nn,3);
	ColdUtility(n).c     = InputData(nn,4);
    if interpri > 0
	ColdUtility(n)
	disp (pausetext)
	pause
    end
end


%	********************************************
%	Reading storage fluid parameters 
%	********************************************


for o = 1:O
	oo = o + N + M + m_index_start;
	StorageFluid(o).full_name =  char ( str_name(oo)' );
	StorageFluid(o).code_name =  char ( str_code(oo)' );
	StorageFluid(o).T_min_SF =  InputData(oo,1);
	StorageFluid(o).T_max_SF =  InputData(oo,2);
	StorageFluid(o).rho_SF   =  InputData(oo,3);
	StorageFluid(o).cp_SF    =  InputData(oo,4);
	StorageFluid(o).h_SF     =  InputData(oo,5);
	StorageFluid(o).c_SF     =  InputData(oo,6);
    if interpri > 0
	StorageFluid(o)
	disp (pausetext)
	pause
    end
end

disp ( [ ' ProcessName = ', ProcessName ] );
disp ( [ ' RunName     = ', RunName     ] );

if interpri > 0
	disp(' ');
	disp(' ');
	disp ( ' Examples of Stream parameters : ' );
	disp ( ' ******************************* ' );
	disp(' ');
	for i=1:I
		disp([int2str(i),'   Heat capacity flow-rate for ',Stream(i).full_name,' = ',num2str(Stream(i).CP),' ', CP.units])		
	end
	for j=1:J
		disp([int2str(j),'   Heat capacity flow-rate for ',Stream(j+I).full_name,' = ',num2str(Stream(j+I).CP),' ',CP.units])
	end
	for m=1:M
		disp([int2str(m),'   Output temperature for ',HotUtility(m).full_name,' = ',num2str(HotUtility(m).T_out),' ',Temp.units])
	end
	for n=1:N
		disp([int2str(n),'   Heat transfer film coefficient for ',ColdUtility(n).full_name,' = ',num2str(ColdUtility(n).h),' ',h.units])
	end
end

%	********************************************
%	Reading HEXs & HSUs Cost Function Parameters 
%	********************************************

n_T_index_start = N + M + m_index_start + 2;
N_XT = InputData(n_T_index_start,1);
N_ST = InputData(n_T_index_start,2);

disp(' ');
Message = '			Now the cost data are assigned to units arrays !';
disp ( Message );
disp(' ');

for n_XT = 1:N_XT
	nn_XT = n_XT + n_T_index_start;
	HEXCostFunction(n_XT).full_name = char ( str_name(nn_XT)' );
	HEXCostFunction(n_XT).code_name = char ( str_code(nn_XT)' );
	HEXCostFunction(n_XT).c0_X  = InputData(nn_XT,1);
	HEXCostFunction(n_XT).c1_X  = InputData(nn_XT,2);
	HEXCostFunction(n_XT).m_X    = InputData(nn_XT,3);
	HEXCostFunction(n_XT).A_min = InputData(nn_XT,4);
	HEXCostFunction(n_XT).Un_X  = InputData(nn_XT,5);
    if interpri > 0
	HEXCostFunction(n_XT)
	disp(' ');
    end
end

for n_ST = 1:N_ST
	nn_ST = n_ST + N_XT + n_T_index_start;
	HSUCostFunction(n_ST).full_name = char ( str_name(nn_ST)' );
	HSUCostFunction(n_ST).code_name = char ( str_code(nn_ST)' );
	HSUCostFunction(n_ST).c0_S  = InputData(nn_ST,1);
	HSUCostFunction(n_ST).c1_S  = InputData(nn_ST,2);
	HSUCostFunction(n_ST).m_S   = InputData(nn_ST,3);
	HSUCostFunction(n_ST).V_min = InputData(nn_ST,4);
   
  	if interpri > 0
		HSUCostFunction(n_ST)
		disp (pausetext)
		pause
  	end
end
  
Ut_X  = InputData(N_ST + N_XT + n_T_index_start+1,1);
disp(' ');
disp ( [ ' Utility HEX factor : ', num2str(Ut_X) ] );
disp(' ');

if interpri > 0
	disp(' ');
	disp ( ' Examples of cost parameters : ' );
	disp ( ' ******************************* ' );
	disp(' ');
	for n_XT=1:N_XT
		disp([int2str(n_XT),'   Process HEX units factor for ',HEXCostFunction(n_XT).full_name,' Un_X = ',num2str(HEXCostFunction(n_XT).Un_X),' ', U.units])
	end
	for n_ST=1:N_ST
		disp([int2str(n_ST),'   Reference cost for ',HSUCostFunction(n_ST).full_name,' c1_S = ',num2str(HSUCostFunction(n_ST).c1_S),' ',cost.units])
	end
	for n_ST=1:N_ST
		disp([int2str(n_ST),'   Fixed cost for ',HSUCostFunction(n_ST).full_name,' cO_S = ',num2str(HSUCostFunction(n_ST).c0_S),' ',cost.units])
	end
end

%	********************************************
%	Reading GA Parameters 
%	********************************************

disp(' ');
Message = '			Now the GA parameters are assigned to units arrays !';
disp ( Message );
disp(' ');

iga = N_ST + N_XT + n_T_index_start + 3;		% une de plus a cause de la ligne Ut_X

HSULevelParam.I_SL     = InputData(iga,1);
HSULevelParam.G_SL     = InputData(iga,2);
HSULevelParam.p_m_SL   = InputData(iga,3);
HSULevelParam.p_c_SL   = InputData(iga,4);
HSULevelParam.S_SL     = InputData(iga,5);

HEXLevelParam.I_XL     = InputData(iga+1,1);
HEXLevelParam.G_XL     = InputData(iga+1,2);
HEXLevelParam.p_m_XL   = InputData(iga+1,3);
HEXLevelParam.p_c_XL   = InputData(iga+1,4);
HEXLevelParam.S_XL     = InputData(iga+1,5);

if interpri > 0
	disp(' ');
	DispNum ( HSULevelParam.I_SL,   'HSULevelParam.I_SL',   'individuals' );
	DispNum ( HSULevelParam.G_SL,   'HSULevelParam.G_SL',   'generations' );
	DispNum ( HSULevelParam.p_m_SL, 'HSULevelParam.p_m_SL', '' );
	DispNum ( HSULevelParam.p_c_SL, 'HSULevelParam.p_c_SL', '' );
	DispNum ( HSULevelParam.S_SL,   'HSULevelParam.S_SL',   '' );
	DispNum ( HEXLevelParam.I_XL,   'HEXLevelParam.I_XL',   'individuals' );
	DispNum ( HEXLevelParam.G_XL,   'HEXLevelParam.G_XL',   'generations' );
	DispNum ( HEXLevelParam.p_m_XL, 'HEXLevelParam.p_m_XL', '' );
	DispNum ( HEXLevelParam.p_c_XL, 'HEXLevelParam.p_c_XL', '' );
	DispNum ( HEXLevelParam.S_XL,   'HEXLevelParam.S_XL',   '' );
end

disp(' ');
disp ( ' End of Program --- InputData !' );
% ***************************************************