
function tplot()
p(1) = subplot('position', [0.05 0.55 0.4 0.4]);
p(2) = subplot('position', [0.05 0.05 0.4 0.4]);
p(3) = subplot('position', [0.55 0.55 0.4 0.4]);
p(4) = subplot('position', [0.55 0.05 0.4 0.4]);

npoints = 4;
nvars = 2;
nobjectives = 2;

points = rand(npoints, nvars) * 100 - 50;
dominated = zeros(npoints, 1);
dying = zeros(1,npoints);				% new K122
neighbours = ones(npoints, 1);
failed_iterations = 0;

a =[-0.581479779325832 -0.37426947600907 -0.303276079487954 -0.14570131437332;0.497803431016739 -0.703289738596115 0.181808789002371 -0.131281024874781;670.06987468506 833.85592099626 1522.33128616535 319.98110459724;4586.74138838826 -2208.70760638392 5145.89940153151 9892.36844745545;0.771130801881329 0.744169954651894 0.716402824257725 -0.5621465124564];

%clear a;
%a(1, :) = -rand(1, 4);
%a([2 4 5], :) = 2.0 * rand(3, 4) - 1.0;
%a(3, :) = rand(1, 4) * 2000;
%a(4, :) = 20000.0 * rand(1, 4) - 10000.0;
%mat2str(a)

obj = objective(points, a);

point_style = ['b+'; 'b+'; 'gx'; 'ro'];
point_style(1, :);

failed = 0;
use = logical(ones(npoints, 1));

for i = 1:10000000
    fprintf('   Population : %3d, Parents : %2d, Bred : %3d, Dying : %2d, Failed : %2d, Failure Rate : %6.4f\n', npoints, sum(use), i, sum(dying), failed, failed / i);
	
	[bred, new, new_obj, p_index, use] = breed(points, obj, use, a);
	if (~bred)
		failed = failed + 1;
	end

	if (i/8 == floor(i/8))
    	dying = kill(points, obj);
	else
		dying = zeros(npoints, 1);
	end

	draw(points, obj, p_index, dying, bred, new, new_obj, a, p);
	
end

function [worked, new, new_obj, p_index, new_use] = breed(points, obj, use, a)
	npoints = size(points, 1);
	ndimensions = size(points, 2);
	nobjectives = size(obj, 2);

	min_obj = min(obj);
	max_obj = max(obj);
	limit_obj = 2 * min_obj - max_obj;
	
	if (sum(use) < nobjectives + ndimensions)
		use = logical(ones(npoints, 1));
	end
	
	thing = 1:npoints;
	indexes = thing(use);
	p_index(1) = indexes(floor(rand(1) * length(indexes)) + 1);
    use(p_index(1)) = logical(0);

	avoid = zeros(npoints, 1);
	avoid(p_index(1)) = 1;
	near_parent = near(points, points(p_index(1), :), avoid, 1);

	if (rand(1) < 0.5 | sum(near_parent) == 0)
		indexes = thing(use);
	else
		indexes = thing(near_parent);
	end

	p_index(2) = indexes(floor(rand(1) * length(indexes)) + 1);
    use(p_index(2)) = logical(0);
	
	avoid = zeros(npoints, 1);
	avoid(p_index(2)) = 1;

	use = use & ~near_parent;
	use = use & ~near(points, points(p_index(2), :), avoid, 1);

    p_points = points(p_index, :);
	p_obj = obj(p_index, :);

	distance = p_points(2, :) - p_points(1, :);

	error = norm(distance) * 0.01;
	error = error * rand(size(distance)) * 2.0 - 1.0;
	error = 0;
	
	middle = (p_points(2, :) + p_points(1, :)) ./ 2;
	weight = randn(1) * 0.5;

	new = middle + distance .* weight + error;
	new_obj = objective(new, a);

	new_max = sum(new_obj > max_obj);
	new_min = sum(new_obj < limit_obj);
	if (new_max > 1 | new_min == 0)
		worked = 1;
	else
		worked = 0;
	end

	new_use = use;

function dying = kill(points, obj)
	npoints = size(points, 1);
	ndimensions = size(points, 2);
	nobjectives = size(obj, 2);

	dominated = logical(zeros(npoints, 1));
	use = ones(npoints, 1);
	while (sum(use) > nobjectives + ndimensions)
		
		thing = 1:npoints;
		indexes = thing(use);
    	an_index = indexes(floor(rand(1) * length(indexes)) + 1);
		
		avoid = zeros(npoints, 1);
		avoid(an_index) = logical(1);
		[closest, close_index, enclosing] = near(points, points(an_index, :), avoid, 1);
		use = use & ~closest;
	
		for j = 1:sum(closest)
			if (dominated(close_index(j)) == 0)
				for k = j+1:sum(closest)
					if (dominated(close_index(k)) == 0)
						dom_test = sum(obj(close_index(j), :) > obj(close_index(k), :));
						if (dom_test == nobjectives)
							dominated(close_index(k)) = logical(1);
						elseif (dom_test == 0)
							dominated(close_index(j)) = logical(1);
							break;
						end
					end
				end
			end
		end
	end
	dying = dominated;

function draw(points, obj, parents, dying, bred, new, new_obj, a, p)

	if (sum(dying) > 0 | bred)
		pmin = min(points);
		pmax = max(points);
		
		int = pmax - pmin;
		pmin = pmin - 0.05 .* int;
		pmax = pmax + 0.05 .* int;
		
		x1 = linspace(pmin(1), pmax(1), 30);
		x2 = linspace(pmin(2), pmax(2), 30);
		[x1, x2] = meshgrid(x1, x2);
		x = cat(3, x1, x2);
		y = objective(x, a);
	
		for j = 1:2
			subplot(p(j));
			hold off;
			surf(x(:, :, 1), x(:, :, 2), y(:, :, j));
			the_axes = axis;
			min_z = the_axes(5);
			hold on;
			
			% Plot points
			plot3(points(:, 1), points(:, 2), obj(:, j), 'b+');
			plot3(points(:, 1), points(:, 2), ones(size(points(:, 1))) * min_z, 'b+');

			% Plot dying points
			if (sum(dying) > 0)
				plot3(points(dying, 1), points(dying, 2), obj(dying, j), 'ro');
			 	plot3(points(dying, 1), points(dying, 2), min_z * ones(sum(dying), 1), 'ro');
			end

			% Plot parents and child
			if (bred)
				% Parents
				plot3(points(parents, 1), points(parents, 2), obj(parents, j), 'g+');
				plot3(points(parents, 1), points(parents, 2), min_z * ones(size(parents)), 'g+');
				plot3(points(parents, 1), points(parents, 2), min_z * ones(size(parents)), 'g-');

				% Child
				plot3(new(1), new(2), new_obj(j), 'g*');
				plot3(new(1), new(2), min_z, 'g*');
			end
		end


		subplot(p(3));
		hold off
		plot(obj(:, 1), obj(:, 2), 'b+');
		hold on
		if (sum(dying) > 0)
			plot(obj(dying, 1), obj(dying, 2), 'ro');
		end
		if (bred)
			plot(obj(parents, 1), obj(parents, 2), 'g+');
			plot(obj(parents, 1), obj(parents, 2), 'g-');
			plot(new_obj(1), new_obj(2), 'k*');
		end

		subplot(p(4));
		hold off
		plot(points(:, 1), points(:, 2), 'b+');
		hold on
		if (sum(dying) > 0)
			plot(points(dying, 1), points(dying, 2), 'ro');
		end
		if (bred)
			plot(points(parents, 1), points(parents, 2), 'g+');
			plot(points(parents, 1), points(parents, 2), 'g-');
			plot(new(1), new(2), 'k*');
		end
	
		pause(0.01)
%		pause
	end
