//////////////////////////////////////////////////////////////////////
// MLGAMain.cpp: implementation of the MLGAMain class.
//
// see MLGAMain.h for explanation header
//
//  Optimize v4.12 - Ka27
//////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include "StruggleH.h"
#include <cmath>
#include <string.h>
// Definition of Genome private data
#include "MLGAMain.h"
#include "mlio.h"


#define SETTINGS_FILE "settings.txt"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

MLGAMain::MLGAMain()
{

}

MLGAMain::~MLGAMain()
{

}

///////////////////////////////////////////////////////
// Main entry point from MATLAB
///////////////////////////////////////////////////////
void MLGAMain::Main(int nlhs,
				 mxArray *plhs[], 
				 int nrhs,
				 const mxArray *prhs[] )
{
	MLInterface UserData;
	MLInterface Parametres;
	MLInterface ControlData;
	
	const int	dims = 1;
	const int	OF_SideDim=5;
	int		inXsize, inDsize, inPTra, inPnPlot;
	int		i, Xm, Xn, Dm, Dn;
	double	OptOBJfun, OF_SideNum, HR_best;
	double  OF_SideFun[OF_SideDim];
	double  *P_OF_SideFun = OF_SideFun;
	mxArray	*array_ptr;

	mexPrintf( "\n   Start MEX interface optimize.dll v4.12 (MATLAB <-> VC++) - 27.10.2000  \n" );
    // verify in/out arguments
	if( nlhs != 3 )
		mexErrMsgTxt("Output must be Genome, Result, FOpt(Result)\n");
	if( nrhs != 3 )
		mexErrMsgTxt("Input must be UserData, Parameters, ControlData\n");

	UserData.SetArgs(prhs[0], true);
	UserData.PrintArg();
	Parametres.SetArgs(prhs[1]);
	Parametres.PrintArg();
	ControlData.SetArgs(prhs[2]);
	ControlData.PrintArg();
	
	printf("Trace = %i\n\n", MLIO::PrintTrace());
	


	/* Unto here */
	mexPrintf( "   Examine input and output arguments in MLGAMain : Stop\n" );

	// Read arrays and sizes
	inXsize =    (int) UserData.ReadDouble(sXsize);
				       UserData.ReadArray(sXopt,&Xm,&Xn);
	inDsize =    (int) UserData.ReadDouble(sDsize);
				       UserData.ReadArray(sDwei,&Dm,&Dn);
	if (Xm*Xn!=inXsize) 
		mexPrintf ( " WARNING ! wrong size for Xval : Xm*Xn!=inXsize %d %d %d", Xm,Xn,inXsize );
	if (Dm*Dn!=inDsize) 
		mexPrintf ( " WARNING ! wrong size for Dwei : Dm*Dn!=inDsize %d %d %d", Dm,Dn,inDsize );

	mexPrintf( "\n******************************************************\n" );
	mexPrintf( " GO : Call of Calculate in MLGAMain, from Main\n" );
	mexPrintf( "******************************************************\n\n" );

	OptOBJfun = Calculate( &HR_best, &UserData, &Parametres, &ControlData );

	mexPrintf( "******* Just after Calculate *********************************\n" );

	OF_SideNum    = OF_SideDim;
	OF_SideFun[0] = 11111.0;
	OF_SideFun[1] = 22222.0;
	OF_SideFun[2] = 33333.0;
	OF_SideFun[3] = OptOBJfun;
	OF_SideFun[4] = HR_best;

//	mexPrintf( "**************************************************************************\n" );
	mexPrintf( " Returning from Calculate, at the end of Main MEX start program !\n" );
/*	mexPrintf ( " Best genome values   inXval[] =\n" );
	mexPrintf( "**** Start UserData.PrintArg()  ******************************************\n" );
	UserData.PrintArg();
	mexPrintf( "**** End of UserData.PrintArg() ******************************************\n" );
	mexPrintf( "\n" );
	mexPrintf ( " Cost function 1 : OF_SideFun[0] = %lf\n", OF_SideFun[0] );
	mexPrintf ( " Cost function 2 : OF_SideFun[1] = %lf\n", OF_SideFun[1] );
	mexPrintf ( " Cost function 3 : OF_SideFun[2] = %lf\n", OF_SideFun[2] );
	mexPrintf ( " Best OF         : OF_SideFun[3] = %lf\n", OF_SideFun[3] );
	mexPrintf ( " HR for best OF  : OF_SideFun[4] = %lf\n", OF_SideFun[4] );
	mexPrintf ( " OptOBJNum       : OF_SideNum    = %lf\n", OF_SideNum );
	mexPrintf( "&&&&&&&&&&&\n" );
*/	mexPrintf ( " P_OF_SideFun[0] = %lf\n", P_OF_SideFun[0] );
	mexPrintf ( " P_OF_SideFun[1] = %lf\n", P_OF_SideFun[1] );
	mexPrintf ( " P_OF_SideFun[2] = %lf\n", P_OF_SideFun[2] );
	mexPrintf ( " P_OF_SideFun[3] = %lf\n", P_OF_SideFun[3] );
	mexPrintf ( " P_OF_SideFun[4] = %lf\n", P_OF_SideFun[4] );
	mexPrintf( "\n" );
		
	// return User Data in [0]
	plhs[0] = UserData.GetArgs();

	// return Objective function in 1
	plhs[1]=mxCreateDoubleMatrix(1,1,mxREAL);
    *mxGetPr(plhs[1]) = OF_SideFun[3];
	
	// return best value in 2
	plhs[2]=mxCreateDoubleMatrix(1,1,mxREAL);
	*mxGetPr(plhs[2]) = OF_SideNum;		// output parameters !!!!!!!!!

}

///////////////////////////////////////////////
// Print info about parametres
///////////////////////////////////////////////
void MLGAMain::PrintParList( GAParameterList& ParList )
{
	int i;

	for( i = 0 ; i < ParList.size(); i++ )
		{
		printf( "%s (%s) =",ParList[i].fullname(), ParList[i].shrtname());
		switch( ParList[i].type())
			{
			case GAParameter::BOOLEAN:
				printf( "boolean %s",*(bool *)ParList[i].value()?"True":"False");
				break;
			case GAParameter::CHAR:
				printf( "char %c",*(char *)ParList[i].value());
				break;
			case GAParameter::STRING:
				printf( "string '%s'",ParList[i].value());
				break;
			case GAParameter::INT:
				printf( "int %d",*(int * )ParList[i].value());
				break;
			case GAParameter::FLOAT:
				printf( "float %f",*(float * )ParList[i].value());
				break;
			case GAParameter::DOUBLE:
				printf( "double %lf",*(double*)ParList[i].value());
				break;
			case GAParameter::POINTER:
				printf( "pointer %lx",ParList[i].value());
		} // Switch	

		printf( "\n" );
	} // For
	printf( "\n" );
} // PrintParList

///////////////////////////////////////////////////
//	To update selected parameters 
///////////////////////////////////////////////////
void MLGAMain::NewParam(GAParameterList *p,
						MLInterface *Parametres )
{
	int inN_POP = (int)Parametres->ReadDouble(sN_POP);
	int inN_GEN = (int)Parametres->ReadDouble(sN_GEN);
	float inP_MUT = Parametres->ReadDouble(sP_MUT);
	float inP_CRO = Parametres->ReadDouble(sP_CRO); 
	
	Parametres->PrintArg();

	p->add(gaNnGenerations, gaSNnGenerations, GAParameter::INT, &inN_GEN);
	p->add(gaNpCrossover, gaSNpCrossover, GAParameter::FLOAT, &inP_CRO);
	p->add(gaNpMutation, gaSNpMutation, GAParameter::FLOAT, &inP_MUT);
	p->add(gaNpopulationSize, gaSNpopulationSize, GAParameter::INT, &inN_POP);

	PrintParList( *p );

} // NewParam


///////////////////////////////////////////////////////////
// Generate population file
///////////////////////////////////////////////////////////
void MLGAMain::write_pop(ostream &fname,
			   StruggleHGA *ga,
			   int ArraySize) 
{
	fname << " Generation = " << ga->generation() << "\t" << "next line : chrom.score, then chrom.real_genome()[count](j=1," << ga->population().size() << ")\n";	
	for(int i=0; i<ga->population().size(); i++)
		{
		MixedGenome &chrom = (MixedGenome &)  ga->population().individual(i);
		MLEvalData *MLEV;
		MLEV = (MLEvalData *) ga->population().individual(i).evalData();
		fname  << " Ind=" << i+1 << "\tOF=" << chrom.score() <<
			"    \tHR_of_OF=" << MLEV->GetGenomeValue ( ArraySize+3 ) << "    \t   with variables : " ;
		for(int j = 0;j<chrom.real_genome().length();++j)
			fname << "  \t" << chrom.real_genome().gene(j);
		fname << "\n";
		}
	
	fname << " Current Objective function in current population = " << ga->statistics().current() << "\n";	
	fname << " Average Objective function in current population = " << ga->population().ave() << "\n";
	fname << " Best    Objective function in current population = " << ga->population().best().score() << "\n\n";
}

//////////////////////////////////////////
// to print out one genome
//////////////////////////////////////////
void MLGAMain::print_genome(ostream &ff,GAGenome& g,MLInterface *d) 
{
	MixedGenome& genome = (MixedGenome&)g;
	static double *x = NULL;
	int countreal,countint,i,m,n;
	double			Xsize = d->ReadDouble(sXsize);
	double			*inXopt = d->ReadArray(sXopt, &m, &n);
	double			*inXval = d->ReadArray(sXval, &m, &n);

	if(!x)
		x= new double[Xsize];

	countreal=0;
	countint =0;
	for(i=0;i<Xsize;++i)
		{
		switch ((int)(10*inXopt[i]))
			{
			case 10 :
				x[i]=genome.int_genome().gene(countint);
				countint++;
				break;
			case 9 :
				x[i]=genome.real_genome().gene(countreal);
				countreal++;
				break;
			case 1 :
				x[i]=inXval[i];			
				break;
			case 0 :
				x[i]=inXval[i];			
				break;
			default :
				printf ( "Wrong value for Xopt (should be 0.0, 0.1, 0.9 or 1.0)\tinXopt[%d]=%lf\n\n\n ***STOP ! ***\n\n",i,inXopt[i] );
				printf("");
			}
		} // For

	for(i=0;i<Xsize;++i) 
		{
		ff<<"x["<<i<<"]="<<x[i]<<"\n";
		}

    ff << " The Objective function for this one best genome  = " << genome.score() << "\n";
}// print_genome


////////////////////////////////////////////////////////////
// Calculate function
////////////////////////////////////////////////////////////
double MLGAMain::Calculate(
					double *HRBest,
					MLInterface *UserData,
					MLInterface *Parametres,
					MLInterface *ControlData)
{
	GAAlleleSetArray ra;
	GAAlleleSetArray na;
	int i;
	int CheckAve, DvarSize;
	double Average_TBC, Best_TBC;
	int PnPlot;
	int i_GEN, nGENtrace;


	// Teatement of Control data
	PnPlot = ControlData->ReadDouble(sPlot);
	if (PnPlot>0)
		{
		nGENtrace = (int)Parametres->ReadDouble(sN_GEN) / PnPlot;
		if (nGENtrace == 0) nGENtrace=1;
		}
	else
		{
		PnPlot= (int)Parametres->ReadDouble(sN_GEN);
		nGENtrace=1;
		printf ("\n WARNING : PNPlot=0 ! Plot will be refreshed, each generation ! ");
		}

	ofstream BestIndOfGen;
	BestIndOfGen.open("BestIndOfGen.log", ios::trunc |ios::out);
	if(BestIndOfGen.bad()) 
		{
		std::cerr<<"Unable to open file 'BestIndOfGen.log'\n";
		exit(0);
		}

	ofstream population;
	population.open("population.txt", ios::trunc |ios::out);
	if(population.bad()) 
		{
		std::cerr<<"Unable to open file '"<<"pop.txt"<<"'\n";
		exit(0);
		}

	MLIO::PrintTrace( (int)Parametres->ReadDouble(sTrace) );		// %&%*%&% Problems ??

	DvarSize = UserData->ReadDouble(sDsize);

	/*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/
	// Output args creation
	int     n_out, n_in;
	mxArray *oa[1], *ia[4];
	n_out = 1;
	n_in  = 4;
	ia[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
	ia[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
	ia[2] = mxCreateDoubleMatrix(1, 1, mxREAL);
	ia[3] = mxCreateDoubleMatrix(1, 1, mxREAL);
	/*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/

	// Adding genome data from arguments arrays 
	double nvals = UserData->ReadDouble(sXsize);
	for(i=0;i<nvals;++i) 
	{
		double *inXopt = UserData->ReadArray(sXopt);
		double *inXmin = UserData->ReadArray(sXmin);
		double *inXmax = UserData->ReadArray(sXmax);
		double *inXwei = UserData->ReadArray(sXwei);
		switch ((int)(10*inXopt[i]))
		{
		case 10 :
			na.add((int)inXmin[i],(int)inXmax[i],GAAllele::INCLUSIVE,GAAllele::INCLUSIVE,inXwei[i]);
			BestIndOfGen <<"optimizing the x integer value \t"<< i<< "\n";
			break;
		case 9 :
			ra.add(inXmin[i],inXmax[i],GAAllele::INCLUSIVE,GAAllele::INCLUSIVE,inXwei[i]);
			BestIndOfGen <<"optimizing the x real    value \t"<< i<< "\n";
			break;
		case 1 :
			BestIndOfGen <<"fixed at initial real    value \t"<< i<< "\n";
			break;
		case 0 :
			BestIndOfGen <<"fixed at initial integer value \t"<< i<< "\n";
			break;
		default :
			BestIndOfGen <<"Wrong value for Xopt (should be 0.0, 0.1, 0.9 or 1.0)\t"<< i<< "\n\n\n ***STOP ! ***\n\n";
			printf ( "Wrong value for Xopt (should be 0.0, 0.1, 0.9 or 1.0)\tinXopt[%d]=%lf\n\n\n ***STOP ! ***\n\n",i,inXopt[i] );
			return(990099);
		}
	} // for
  
	BestIndOfGen<<"\n\nReal AlleleSetArray  \n\n"<<ra<<"\n";
                                                     
	MixedGenome *mg =  new MixedGenome(ra,na,CalcObjective,(void *)UserData);

	/********** $$$$$$$   Start initialisation MLEvalData        $$$$$$$$ ************/
	/*	We are just after the basic definition and creation of our genome
		Now the goal is to add a MLEvalData object to Genome class */

	if (MLIO::PrintTrace() > 1)	printf ( " Start                     ===> Greffe\n");

	MLEvalData EVDAmp;									// Original MLEvalData
	MLEvalData *EVDNew;									// A new pointer
	EVDNew = (MLEvalData *) (mg->evalData (EVDAmp));	// Clone EVDNew with EVDAmp

	printf( "Nb of clones left (Expected 2 !) = %i\n\n",MLEvalData::GetNbClone()); // Expected = 2

	if (MLIO::PrintTrace() > 1)	
		printf ("Address EVDNew & EVDAmp %d %d\n",EVDNew,EVDAmp);
	if (EVDNew == NULL) 
		printf ( " EVDNew == NULL   $$-$$\n");
	if (MLIO::PrintTrace() > 1)	
		printf ( " - Stick MLEvalData on genome in Calculate ! EVDNew->GetNbClone() = %d\n",EVDNew->GetNbClone() );
	if (MLIO::PrintTrace() > 1)	
		printf ( " Stop                      ===> Greffe\n\n");
	/********** $$$$$$$   End  initialisation MLEvalData         $$$$$$$ ************/

	mg->comparator(my_comparator);						// Definitions
	mg->crossover(MixedGenome::MixedCrossover);
//	mg->crossover(GARealBlendCrossover);

	printf( "Nb of clones created  = %i\n",MLEvalData::GetNbClone()); 
	printf ( " Before Struggle definition\n");
	StruggleHGA *hga = new StruggleHGA((GAGenome &)*mg);
	printf ( " After  Struggle definition\n");
	printf( "Nb of clones left (Expected 2 ?) = %i\n",MLEvalData::GetNbClone()); 

	// modifie les parametres
	GAParameterList *ParList;
	if (MLIO::PrintTrace() > 0)	
		printf ( "\n Parameter List\n");
  
	if (MLIO::PrintTrace() > 1)	
		{
		ParList = new GAParameterList;
		*ParList = hga->parameters();			// default parameter list
		PrintParList( *ParList );
		delete ParList;
		}

	hga->parameters(SETTINGS_FILE);
  
	if (MLIO::PrintTrace() > 1)	
		{
		ParList = new GAParameterList;
		*ParList = hga->parameters();			// new values read from file
		PrintParList( *ParList );
		delete ParList;
		}

	ParList = new GAParameterList;
	NewParam( ParList,Parametres);	
	// actualise les donnes des parametres
	hga->parameters(*ParList);
	if (MLIO::PrintTrace() > 0)
	{
		*ParList = hga->parameters();			// new values from passed arguments
		PrintParList( *ParList );
	}
	delete ParList;

	printf( "Nb of clones left = %i\n\n",MLEvalData::GetNbClone()); 

	printf( "\n\n*****************************************************\n" );
	printf( "In Calculate (begin hga loop)" );
	printf( "\n*****************************************************\n\n" );
  

	printf("* hga->minimize();\n");
	hga->minimize();
	printf("* hga->initialize();\n");
	hga->initialize();

	printf("* Before while(!hga->done()) ?\n");
	while(!hga->done() )
		{
		printf("Take the best");
		MixedGenome & chrom = (MixedGenome &)(hga->population().best());
		i_GEN = hga->generation();
		if ( (i_GEN / nGENtrace)*nGENtrace == i_GEN )
				write_pop(population,hga,DvarSize);
		BestIndOfGen << " Generation no " << i_GEN << ", the best genome in this population is \n";
		print_genome ( BestIndOfGen, chrom, UserData );
		Average_TBC = hga->population().ave();
		Best_TBC    = hga->population().best().score();
		BestIndOfGen << " Average Objective function in current population = " << Average_TBC << "\n\n";
		if (MLIO::PrintTrace() > 0)	
			printf( "\nScore of the best genome    : hga->population().best().score() = %f\n", Best_TBC );
		if (MLIO::PrintTrace() > 0)	
			printf( "Average score of population : hga->population().ave() = %f\n", Average_TBC );
		/*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/
		if (MLIO::PrintTrace() > 1)	
			printf ( " Just before mexCallMATLAB(n_out, oa, n_in, ia, ""RAve"" )\n" );
		MLEvalData *MLEV;
		MLEV = (MLEvalData *) hga->population().best().evalData();
		*mxGetPr(ia[0]) = Average_TBC;
		*mxGetPr(ia[1]) = MLEV->GetGenomeValue ( DvarSize+4 );
		*mxGetPr(ia[2]) = MLEV->GetGenomeValue ( DvarSize+3 );
		*mxGetPr(ia[3]) = 0;

		if (MLIO::PrintTrace() > 0)	
			printf( "Nb of clones before = %i\n", MLEV->GetNbClone() );

		mexCallMATLAB(n_out, oa, n_in, ia, "RAve" );
		
		if (MLIO::PrintTrace() > 1)	
			printf ( " Just after  mexCallMATLAB(n_out, oa, n_in, ia, ""RAve"" )\n\n" );

		if (MLIO::PrintTrace() > 0)	
			printf( "Nb of clones after = %i\n", MLEV->GetNbClone() );

		CheckAve = (int) mxGetScalar(oa[0]);
		if (CheckAve != 989) 
			mexErrMsgTxt ( " ERROR in callto MATLAB RAve.m ! Check ! " );
		mxDestroyArray( oa[0] );

		/*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/
		if ( (i_GEN / nGENtrace)*nGENtrace == i_GEN )
				hga->statistics().write(BestIndOfGen);

		hga->step();
	} // while

    MixedGenome & chrom = (MixedGenome &)(hga->population().best());
    write_pop(population,hga,DvarSize);
	i_GEN = hga->generation();
    BestIndOfGen << "\n ====>> Generation no " << i_GEN << ", the best genome in this population is \n";
	print_genome ( BestIndOfGen, chrom, UserData );
	Average_TBC = hga->population().ave();
	Best_TBC    = hga->population().best().score();
	BestIndOfGen << " Average Objective function in current population = " << Average_TBC << "\n\n";
	BestIndOfGen << " Statistics of the current population :\n";
	if (MLIO::PrintTrace() > 0)	
		printf( "\nScore of the best genome  : hga->population().best().score() = %f\n", Best_TBC );
	if (MLIO::PrintTrace() > 0)	
		printf( "Average score of population : hga->population().ave() = %f\n", Average_TBC );
	/*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/
	if (MLIO::PrintTrace() > 1)
		printf ( " Just before mexCallMATLAB( n_out, oa, n_in, ia, ""RAve"" )\n" );

	MLEvalData *MLEV;
	MLEV = (MLEvalData *) (hga->population().best().evalData());
	*mxGetPr(ia[0]) = Average_TBC;
	*mxGetPr(ia[1]) = MLEV->GetGenomeValue ( DvarSize+4 );
	*mxGetPr(ia[2]) = MLEV->GetGenomeValue ( DvarSize+3 );

	
	*HRBest = MLEV->GetGenomeValue ( DvarSize+3 );		//	!?! verify !
	
	*mxGetPr(ia[3]) = 1;	// Special treatment in Rave for last call
	
	mexCallMATLAB(n_out, oa, n_in, ia, "RAve" );

	if (MLIO::PrintTrace() > 1)	
		printf ( " Just after  mexCallMATLAB(n_out, oa, n_in, ia, ""RAve"" )\n\n" );

	CheckAve = (int) mxGetScalar(oa[0]);
	if (CheckAve != 989) 
		mexErrMsgTxt ( " ERROR in callto MATLAB RAve.m ! Check ! " );
	mxDestroyArray( oa[0] );
	mxDestroyArray( ia[0] );	// At the final end only !
	mxDestroyArray( ia[1] );
	mxDestroyArray( ia[2] );
	mxDestroyArray( ia[3] );
	/*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/
	hga->statistics().write(BestIndOfGen);

	//	GAStatistics Mean;
	if (MLIO::PrintTrace() > 1)	
		printf( "In Calculate End : hga->statistics().current(GAStatistics::Mean) = %f\n",
									hga->statistics().current(GAStatistics::Mean) );

	population.close();
	BestIndOfGen.close();

	printf( "\n\nEnd1 - Nb of clones left = over %i\n\n",MLEvalData::GetNbClone());

	//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	//	Return the best genome in inXval input array
	int countreal,countint;
	double MaxCount;
	double *OptArray;
	double *ValArray;

	MaxCount = UserData->ReadDouble(sXsize);	//	Total number of gene
	OptArray = UserData->ReadArray(sXopt);		//	optimization parameters
	ValArray = UserData->ReadArray(sXval);		//	initial values (real, integer)
	
	printf("MaxCount = %f\n", MaxCount );
	for(i=0,countreal=0,countint=0; i<(int)MaxCount; ++i) 
	{
		if		(OptArray[i] == 0.9)			// if this gene is in the real genome
		{
			ValArray[i]=chrom.real_genome().gene(countreal++);
			printf("OptArray[%2d] = %f  --  ValArray[%2d] = %f\n", i, OptArray[i], i, ValArray[i] );
		}
		else if	(OptArray[i] == 1.0)			// if this gene is in the integer genome
		{
			ValArray[i]=chrom.int_genome().gene(countint++);
			printf("OptArray[%2d] = %f  --  ValArray[%2d] = %f\n", i, OptArray[i], i, ValArray[i] );
		}
		else if	(OptArray[i] == 0.1)			// if this gene is in the real genome
			printf("OptArray[%2d] = %f\n", i, OptArray[i] );
		else if	(OptArray[i] == 0.0)			// if this gene is in the integer genome
			printf("OptArray[%2d] = %f\n", i, OptArray[i] );
		else									// if not
			printf ( "Wrong value for Xopt (should be 0.0, 0.1, 0.9 or 1.0)\tinXopt[%d]=%lf\n\n\n ***STOP ! ***\n\n",i,OptArray[i] );
	}
//	printf("--------------------------\n");
	/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
	// Clean up
	delete hga;
//	printf( "End2 - Nb of clones left = over %i\n",MLEvalData::GetNbClone());
	delete mg;									//	see if nvarInt = 0 ?  ==> erreur !
//	printf( "End3 - Nb of clones left = over %i\n",MLEvalData::GetNbClone());

	return(Best_TBC);
} // Calculate
 
