/* ----------------------------------------------------------------------------
  gamain.cpp
  AM
  BR K316 - K429
  Optimize v3.92 - K524
---------------------------------------------------------------------------- */
#include <stdio.h>
#include <cmath>
#include <string.h>
#include "MLGAMain.h"
#include "mlio.h"

#define SQR(x) ((x)*(x))

///////////////////////////////////////////////////////
// Main entry point from mathlab
///////////////////////////////////////////////////////
void mexFunction(int nlhs,
				 mxArray *plhs[], 
				 int nrhs,
				 const mxArray *prhs[] )
{
	MLGAMain	GAMain;

	GAMain.Main(nlhs, plhs, nrhs, prhs);

}// Calculate OBJ the objective function in call to ROFm.m

double simul_OF_ (double *CurrentGenome,	// Genome
				int TailleGenome,		// Genome size
				GAGenome& g)
{
    int     i;
	int		GTkGTsize;
 	int     n_out, n_in;
	mxArray *oa[3], *ia[3];
	double  *GTkGT, *o_OFset;
	int		mrow0, ncol0, mrow1, ncol1, mrow2, ncol2;

	if (MLIO::PrintTrace() > 1)	
		printf( "\n\n simul_OF_ : Call of OFm : CurrentGenome[1] = %f - size genome = %d\n", 
				CurrentGenome[1], TailleGenome );

	n_out = 3;
	n_in  = 3;

	// Input array initialization
	ia[0] = mxCreateString( "Genome from simul_OF_ !" );
	ia[1] = mxCreateDoubleMatrix( 1, TailleGenome, mxREAL);
	memcpy( mxGetPr(ia[1]), CurrentGenome, TailleGenome*sizeof(double) );
	ia[2] = mxCreateDoubleMatrix(1, 1, mxREAL);
	*mxGetPr(ia[2]) = TailleGenome;

	/*  check for proper number of arguments */
	if(n_in!=3) 
		mexErrMsgTxt(" 3 inputs required.");
	if(n_out!=3) 
		mexErrMsgTxt(" 3 output required.");
  
	if (MLIO::PrintTrace() > 1)	
		printf ( " Just before mexCallMATLAB(n_out, oa, n_in, ia, ""ROFm"" )\n" );
	
	// Call mathlab Objective function
	mexCallMATLAB(n_out, oa, n_in, ia, "ROFm" );
	
	if (MLIO::PrintTrace() > 1)	
		printf ( " Just after  mexCallMATLAB(n_out, oa, n_in, ia, ""ROFm"" )\n\n" );

	/*  get the dimensions of the matrix input y */
	mrow0 = mxGetM(oa[0]);
	ncol0 = mxGetN(oa[0]);
	mrow1 = mxGetM(oa[1]);
	ncol1 = mxGetN(oa[1]);
	mrow2 = mxGetM(oa[2]);
	ncol2 = mxGetN(oa[2]);

	if (MLIO::PrintTrace() > 2)		
		printf ( "Size parameters :  mrow0, ncol0, mrow1, ncol1, mrow2, ncol2 =   %d %d %d %d %d %d\n",  mrow0, ncol0, mrow1, ncol1, mrow2, ncol2 );

	if(mrow0!=1 || ncol0!=5)	
		mexErrMsgTxt( " simul_OF_ Wrong array dimension for output argument 1 : " );

	o_OFset = mxGetPr(oa[0]);    //	c_HEN, c_STO, c_UTIL, HR, TBCs	

	if (MLIO::PrintTrace() > 1)	
		{
		printf( "    Output argument #1 : o_OFset( c_HEN, c_STO, c_UTIL, HR, TBCs ) = " );
		for ( i = 0; i < ncol0; i++ )
			printf( " %lf", o_OFset[i]);
		printf( "\n" );
		}

	// test to get correct size of array GTkGT
	if(mrow2!=1 || ncol2!=1)	
		mexErrMsgTxt( " simul_OF_ Wrong array dimension for output argument 3 : " );
	GTkGTsize = (int) mxGetScalar(oa[2]);
	if (MLIO::PrintTrace() > 1)	
		printf ( "System parameters :   GTkGTsize =   %d\n\n", GTkGTsize );
	if(mrow1!=1 || ncol1!=GTkGTsize)	
		mexErrMsgTxt( " simul_OF_ Wrong array dimension for output argument 2 : " );

	// with GTkGT
	GTkGT = (double *) mxGetPr(oa[1]);		//	Super vector, size Dsize

	/* Remove MLEvalData for code test of distance calculation
	/*****************  $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$  ********************/
	/*	In simul_of_ just after the calculation of temperatures and Objective function in ROFm.m
		Aim here is to get the grouped temperatures GTkGT in evalData by SetTemperatureArray	*/

	MLEvalData *EVDT;
	EVDT = ((MLEvalData *) (g.evalData()));
	if (EVDT == NULL)
		printf ( " (MLEvalData *) (g.evalData()) == NULL   $$\n");
	EVDT->SetTemperatureArray(GTkGTsize,GTkGT);
	for ( i = 0; i < ncol0; i++ )
		EVDT->SetTemperatureArrayValue(GTkGTsize+i,o_OFset[i]);

	/*****************  $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$  *********************/
	mxDestroyArray( oa[0] );
	mxDestroyArray( oa[1] );
	mxDestroyArray( oa[2] );
	mxDestroyArray( ia[0] );
	mxDestroyArray( ia[1] );
	mxDestroyArray( ia[2] );

	return o_OFset[4];			// Yes, it's the fifth one !
} // simul_OF_


/////////////////////////////////////////////////////
// Initialisation de _OF_ (Objective function)
/////////////////////////////////////////////////////
void init_OF_ (double *CurrentGenome,
			   double *InitialGenome, 
			   int TailleGenome)
{
	memcpy ( CurrentGenome, InitialGenome, TailleGenome*sizeof(double) );
} // init_OF


//////////////////////////////////////////////////////////
// simple comparator with Temperatures and Weights
//////////////////////////////////////////////////////////
float my_comparator(const GAGenome& g1, 
					const GAGenome& g2) 
{
	MixedGenome		&a = ( MixedGenome &) g1;
	MixedGenome		&b = ( MixedGenome &) g2;
	float			sum=0;
	int				m, n;
	
	MLInterface		*d = (MLInterface *)a.userData();// my population data structure
	double			Dsize = d->ReadDouble(sDsize);
	double			*inDwei = d->ReadArray(sDwei, &m, &n);
	
	int k;
	static int Iloop = 0;
	double TTka[300], TTkb[300];
	double *PtrTTka = TTka;
	double *PtrTTkb = TTkb;

	Iloop += 1;

	/**********************************************************************************
		Aim here is to reread and print the temperatures GTk and GT
		by GetTemperatureArray of MLevalData */

	MLEvalData *a_evda;
	MLEvalData *b_evda;
	a_evda = ((MLEvalData *) (a.evalData()));	// my individual data structure
	b_evda = ((MLEvalData *) (b.evalData()));
	if (a_evda == NULL) 
		printf ( " ((MLEvalData *) (a.evalData())) == NULL   $$\n");
	if (b_evda == NULL) 
		printf ( " ((MLEvalData *) (b.evalData())) == NULL   $$\n");
//	ss_Data *d = (ss_Data *)a.userData();	// my population data structure

	PtrTTka = a_evda->GetTemperatureArray();
	PtrTTkb = b_evda->GetTemperatureArray();

	if (MLIO::PrintTrace() > 1)	
		printf( "**************************** Calculate distance *******************************\n" );

	for (k=0;k<Dsize;k++)
		{
		sum += SQR( inDwei[k] * ( PtrTTka[k]-PtrTTkb[k] ) );	// p = 0 : Dsize-1
		}

	if (MLIO::PrintTrace() > 1)
		printf ( "  Iloop = %d * Distance using Super Temperature Array : sum = %9.5lf\n",Iloop,sum );
	if (MLIO::PrintTrace() > 1)	
		printf( "*********************************** Done **************************************\n\n" );

	return sqrt(sum);
} // my_Comparator


//////////////////////////////////////////////////////////
// Calculate objective function
//////////////////////////////////////////////////////////
float CalcObjective(GAGenome& g) 
{
	MixedGenome& genome = (MixedGenome&)g;
	static double	*x = NULL;
	static double	*kvals=NULL; 
	double			obj=0.0;
	int				cnt,i,m,n;

	MLInterface		*d = (MLInterface *)genome.userData();	// my data structure
	int				Xsize = (int)d->ReadDouble(sXsize);
	double			*inXval = d->ReadArray(sXval,&m,&n);
	double			*inXopt = d->ReadArray(sXopt,&m,&n);

	printf("Call objective function\n val = %li, opt = %li", inXval, inXopt);

	d->PrintArg();

	if(!x) 
		{
		x= new double[Xsize];
		init_OF_(x,inXval,Xsize);				 // Is it usefull ?
		}

	printf("write in x[] init_OF_ XSize = %d\n", Xsize);
	cnt=0;
	for(i=0;i<Xsize;++i) 
		{
		if(inXopt[i]) 
			{    // this parameter is in the gene to be optimized
			printf( "appel du gene\n");
			printf("write in x[] from gene() = %lf\n",genome.real_genome().gene(cnt));
			x[i]=genome.real_genome().gene(cnt);
			cnt++;
			}
		else 
			{				  // it's not, so find it from the original data
			printf("write in x[] from intXval() = %lf\n",inXval[i]);
			x[i]=inXval[i];			
			}
		}

	if (MLIO::PrintTrace() > 1)	
		{
		printf( "************* Middle part of CalcObjective, before simul_of_  ! ******* ???????????????? ******\n" );
		printf (" x[1:%d] = ", Xsize );
		for(i=0;i<Xsize;++i) 
			{
			printf ( " %5.3f",x[i] );
			}
		} // if

	obj = simul_OF_(x,Xsize,g);

#ifdef PRINT_VALUES
	for(i=0;i<Xsize;++i) 
		{
		BestIndOfGen<<"x["<<i<<"]="<<x[i]<<"\n";
		}
	BestIndOfGen<<"obj="<<obj<<"\n";
#endif

  return ((float)obj);
} // CalcObjective
